//-< SOCKIO.H >------------------------------------------------------*--------*
// D-Systemt                 Version 1.0         (c) 1999  GARRET    *     ?  *
// (Simple Distributed System)                                       *   /\|  *
//                                                                   *  /  \  *
//                          Created:     03-Apr-2013    K.A. Knizhnik  * / [] \ *
//                          Last update: 04-Apr-2013    K.A. Knizhnik  * GARRET *
//-------------------------------------------------------------------*--------*
// Socket interface
//-------------------------------------------------------------------*--------*

#ifndef __SOCKIO_H__
#define __SOCKIO_H__

/**
 * Socket interface
 */
class DSSocket
{
  public: 
    enum OpenMode 
    { 
        NoDelay           = 1,
        LocalDomainOnly   = 2,
        GlobalDomainOnly  = 4,
        DoNotReuseAddress = 8
    };
    

    static char const* UnixSocketDir;

    /**
     * Write specfied number of bytes to socket. Throws exception in case of IO error.
     * @param data data to be written
     * @param size size of sent data
     */
    void write(void const* data, size_t size);    

    /**
     * Read specified number of bytes from socket.
     * @param data buffer to receive data
     * @param size number of bytes to read from socket
     */    
    void read(void* data, size_t size);

    /**
     * Accept connection with clients
     * @param clients size of this arrays specifies number of clients
     * @param listen port 
     * @param mode socket mode
     * @param linger time (0 to disable)
     */
    static void accept(std::vector<DSSocket>& clients, int port, int mode = 0, int linger = 0);
    
    /**
     * Connect to server
     * @param host server address
     * @param port server port
     * @param mode socket mode
     * @param nAttempts maximal number of connection attempts
     * @param timeout interval in seconds between reconnect 
     */
    void connect(char const* host, int port, int mode = 0, int nAttempts = 10, int timeout = 1, int linger = 0);

    /**
     * Close connection
     */
    void disconnect();

    DSSocket() : sd(-1) {}
    ~DSSocket();

  private:
    int sd;
};

#endif
